open Util
open Grid

type direction = N | S | E | W

type commandType = Appear of (point * direction list)
                 | Move of direction list
                 | Pick of packageId list
                 | Drop of packageId list

type command = bid * commandType

type event = robot * commandType

let rec goDirections (x, y) dirList =
  match dirList with 
       [] -> (x, y)
    |  (dir::dirs) -> (goDirections (match dir with
      N -> (x+1, y)
    | S -> (x-1, y)
    | E -> (x, y+1)
    | W -> (x, y-1)) dirs)

let rec packageIdList_to_string (id: packageId list):string = String.concat " " (List.map string_of_int id)

let rec packageList_to_string (pkgs: package list):string = String.concat ";" (List.map package_to_string pkgs)

and package_to_string (id, (maybeRobot, (x, y), weight)) = (string_of_int id) ^ " X " ^ (string_of_int x) ^ " Y " ^ (string_of_int y) ^ " " ^ (string_of_int weight)
 
let robot_to_string (myId: int) (robot, ((x, y), packages)) = 
  let str = (string_of_int robot) ^ " X " ^ (string_of_int x) ^ " Y " ^ (string_of_int y) ^ " " ^ "[" ^ (String.concat ";" (List.map string_of_int packages)) ^ "]" in
    (if (robot == myId) then
       "***" ^ str ^ "***"
     else
       str)

let robotList_to_string (myId: int) (robots: robotList):string = String.concat "\n" (List.map (robot_to_string myId) robots)

let direction_to_string dir =
      match dir with
        N -> "N"
     |  S -> "S"      
     |  E -> "E"
     |  W -> "W"

let rec eventList_to_string (events: event list):string = String.concat "\n" (List.map event_to_string events)

and event_to_string (robotId, commandType) = (string_of_int robotId) ^ " " ^ (commandType_to_string commandType)

(* must add 1 to x and y *)
and command_to_string (bid, commandTy) = (string_of_int bid) ^ (commandType_to_string commandTy)

and commandType_to_string (commandTy:commandType):string = 
   (match commandTy with
(* flip the coordinates: we store in row-major, they store in column-major (or possibly vice versa *)
                             Appear ((x, y), dirs) -> "X " ^ (string_of_int (y+1)) ^ " Y " ^ (string_of_int (x+1)) ^ " " ^ (String.concat " " (List.map direction_to_string dirs))
                           | Move dir -> " Move " ^ (String.concat " " (List.map direction_to_string dir))
                           | Pick pkgs -> " Pick " ^ (String.concat " " (List.map string_of_int pkgs))
                           | Drop pkgs -> " Drop " ^ (String.concat " " (List.map string_of_int pkgs)))


let  string_to_direction = function
  "N" -> N
| "S" -> S
| "E" -> E
| "W" -> W
| _ -> raise Util.Fail ("bad direction")

let getRobotPos (robotId:robot) (robots:robotList):point = 
   (* don't need to check for Not_found, since we're handling a Move command *)
   (fst (List.assoc robotId robots))

let makeOwnedBy (robotId:robot) (id, (_, dest, weight):package):package = (id, (Some robotId, dest, weight)) 

let makeUnowned (id, (_, dest, weight):package):package = (id, (None, dest, weight)) 