open Grid
open Util

(******* selectors for the grid data structure *****)

(* given a point in the grid, returns the list of robot id's for that grid *)

let getRobots ((x, y) as p:point) ((grid, _):gameState):robot list = 
   try
     (fst3 (grid.(x).(y))) 
   with Invalid_argument _ -> []

(* given a point in the grid, returns the list of package descriptors for that grid *)

let getPackages ((x, y) as p:point) ((grid, _):gameState):packageList = 
  try
     (snd3 (grid.(x).(y)))
  with Invalid_argument _ -> []
  
(* given a point in the grid, return its type *)

let getType ((x, y) as p:point) ((grid, _):gameState):spaceType = 
  try
    (thrd3 (grid.(x).(y))) 
  with Invalid_argument _ -> Wall

(* given a robot ID, return its location *)

let robotLocation (r: robot) ((_, robots):gameState):point = 
   try
     fst (List.assoc r robots)
   with Not_found -> (-1, -1)

(* given a robot ID, return its packages *)

let robotPackages (r:robot) ((_, robots):gameState):packageId list =
  try
     snd (List.assoc r robots)
  with Not_found -> []

(* given a package ID and its location, return its owner *)

let packageOwner (p: packageId) ((x, y):point) ((grid, _):gameState):robot option = match (grid.(x).(y)) with
     (_, pkgs, _) -> try (match (List.assoc p pkgs) with
                            (own, _, _) -> own)
                     with Not_found -> None

(* given a package ID and its location, return its destination *)

let packageDest (p: packageId) ((x, y):point) ((grid, _):gameState) :point = match (grid.(x).(y)) with
   
     (_, pkgs, _) -> try (match (List.assoc p pkgs) with
                            (_, dest, _) -> dest)
                     with Not_found -> (-1, -1)

(* given a package ID and its location, return its weight *)

let packageWeight (p: packageId) ((x, y):point) ((grid, _):gameState) :int = match (grid.(x).(y)) with
  (_, pkgs, _) -> try (match (List.assoc p pkgs) with
                         (_, _, weight) -> weight)
                  with Not_found -> -1

(* given a list of package IDs and a list of positions, return a list of packages *)

let idsToPackages (ps: packageId list) (locs: point list) (g:gameState): package list =
  let pairs = List.combine ps locs in
    (List.map (fun (id, loc) -> (id, ((packageOwner id loc g), (packageDest id loc g), (packageWeight id loc g)))) pairs)

(* given a game state, return the size of the grid: (width, height) *)

let bounds ((grid, _):gameState):point = (Array.length(grid), Array.length(grid.(0)))

(* given a package, return true if it's currently possessed by a robot *)

let isOwned (p: package):bool = match p with
  (_, (Some _, _, _)) -> true
| _ -> false

let packageOwner (p: package): robot option = match p with
  (_, (own, _, _)) -> own
