open Str
open Grid

exception Fail

let words (s: string):string list = Str.split (regexp "[ \t]+") s

let lines (s: string):string list = Str.split (regexp "[\n]") s

let rec group (n:int) (xs: 'a list): 'a list list = 
(* length xs must = 0 mod n *)
    match xs with
      [] -> []
    | _ -> List.append [(take n xs)] (group n (drop n xs))

and take (n:int) (l: 'a list):'a list =
  if (n == 0) then
     []
  else 
      match l with
         [] -> raise Fail("take: empty list") 
       | (x::xs) -> (x::(take (n - 1) xs))

and drop (n:int) (l: 'a list): 'a list = 
   if (n == 0) then
       l
   else
      match l with
         [] -> raise Fail("drop: empty list")
       | (x::xs) -> (drop (n - 1) xs)

and split3 (triples: ('a * 'b * 'c) list): ('a list * 'b list * 'c list) =
      match triples with
         [] -> ([], [], [])
       | ((x, y, z)::xs) -> let (xss, yss, zss) = split3 xs in (x::xss, y::yss, z::zss)

and fst3 (a, _, _) = a

and snd3 (_, a, _) = a

and thrd3 (_, _, a) = a

let rec explode = function
           "" -> []
         | s  -> (String.get s 0) ::
                 explode (String.sub s 1 ((String.length s) - 1));;

       (* Convert a list of characters to a string *)  
let rec implode = function
           []       -> ""
         | charlist -> (Char.escaped (List.hd charlist)) ^
                       (implode (List.tl charlist));;

let deepCopy (g:grid) = Array.init (Array.length g) (fun rowNum -> Array.copy g.(rowNum))

let rec remove elt lst = match lst with
   (x::xs) when x == elt -> (remove elt xs)
 | (x::xs) -> (x::(remove elt xs))
 | [] -> []

let rec replicate n x = (if (n == 0) then []
                         else (x::(replicate (n-1) x)))

(* prefers the newer element *)
let addNoDups elt l = if (List.mem elt l) then (elt::(remove elt l)) else (elt::l)

let addNoDups_assoc ((key, _) as elt: ('a* 'b)) (l: ('a* 'b) list):('a*'b) list = 
      (if (List.mem_assoc key l) then (elt::(List.remove_assoc key l)) else (elt::l))

let addListNoDups_assoc (elts: ('a* 'b) list) (l: ('a* 'b) list) = 
    (List.fold_left (fun rest elt -> addNoDups_assoc elt rest) l elts)
